﻿local MAJOR, MINOR = "Vengeance", 1;
local Addon = LibStub("AceAddon-3.0"):NewAddon(MAJOR
	,"AceEvent-3.0"
);
-- debug
_G.Vengeance = Addon;

VengeanceSavedVariable = {
	FrameRect = {},       ShowAlways = true,     MouseEnabled = true,
	Opacity = 1,          Texture = nil,         ColorBar = nil,
	ColorBarMarker = nil, ColorBackground = nil, ShowBorder = true,
	BorderWidth = 3,      MaxValue = 0,
};

local UnitBuff, select, unpack, floor, min, max
    = UnitBuff, select, unpack, math.floor, math.min, math.max;
local VENGEANCE_SPELL_ID = 93099;

local function ToKiloNumber(number)
	return (floor(number/100+0.5)/10).."k";
end
local function ToMegaNumber(number)
	return (floor(number/100000+0.5)/10).."M";
end
local function ToPercentNumber(frac)
	return (floor(frac*1000+.5)/10).."%";
end

function Addon:OnEnable()
	-- Get spell name:
	-- Vengeance spell id's: 93099 (DK), 84840 (Druid), 84839 (Paladin),
	--   93098 (Warrior), 120267 (Monk)
	-- All have the same name though, so we only need one, then search for it as
	-- a buff. This way it works on all locales and does not require special
	-- code for each class.
	self.VengeanceName = GetSpellInfo(VENGEANCE_SPELL_ID);
	self.VengeanceMax  = 1;
	self.VengeancePrev = 0; -- Vengeance max value for the *previous* fight
	self.VengeanceCurrent = 0; -- Vengeance max value for the *current* fight

	local SV = VengeanceSavedVariable;
	local SM = LibStub("LibSharedMedia-3.0");
	local Frame = CreateFrame("StatusBar");
	local Border = Frame:CreateTexture(nil, "BACKGROUND"); -- Border texture
	local BG = Frame:CreateTexture(nil, "BORDER");         -- Background texture
	local FG = Frame:CreateTexture(nil, "ARTWORK");        -- "Bar" texture
	local MK = Frame:CreateTexture(nil, "ARTWORK");        -- Marker texture
	local TX = Frame:CreateFontString(nil, "OVERLAY");     -- Text
	self.Frame = Frame;
	
	do -- Set up textures and frames
		Frame:SetParent(UIParent);
		Frame:SetPoint("CENTER");
		Frame:SetWidth(SV.Width or 200);
		Frame:SetHeight(SV.Height or 20);
		Frame:SetAlpha(SV.Opacity or 1);
		Frame:SetOrientation("HORIZONTAL");
		Frame:SetMovable(true);
		Frame:SetMinMaxValues(0, 100);
		Frame:SetValue(0);
		
		Border:SetTexture(0, 0, 0, .3);
		Border:SetPoint("TOPLEFT", Frame, -(SV.BorderWidth or 3), SV.BorderWidth or 3);
		Border:SetPoint("BOTTOMRIGHT", Frame, SV.BorderWidth or 3, -(SV.BorderWidth or 3));

		BG:SetTexture(0, 0, 0, .5);
		BG:SetAllPoints(Frame);

		FG:SetTexture(.8, .6, .1, 1);
		Frame:SetStatusBarTexture(FG);
		
		MK:SetParent(Frame);
		MK:SetTexture(.8, .6, .1, 1);
		MK:SetWidth(3);
		MK:SetPoint("TOP");
		MK:SetPoint("BOTTOM");
		
		TX:SetFontObject(GameFontNormal);
		TX:SetTextColor(.5, .5, .5, 1);
		TX:SetAllPoints(Frame);
		TX:SetJustifyH("CENTER");
		TX:SetJustifyV("MIDDLE");
	end
	
	self.SetTexture = function(self, tx)
		if tx then
			FG:SetTexture(SM:Fetch("statusbar", tx));
		elseif SV.ColorBar then
			FG:SetTexture(unpack(SV.ColorBar));
		end
		SV.Texture = tx;
		Frame:SetStatusBarTexture(FG);
	end
	self.SetColorBar = function(self, r,g,b,a)
		if SV.Texture then
			FG:SetVertexColor(r, g, b, a);
		else
			FG:SetTexture(r, g, b, a);
		end
		Frame:SetStatusBarTexture(FG);
		SV.ColorBar = {r,g,b,a};
	end
	self.SetColorBarMarker = function(self, r,g,b,a)
		if SV.MarkerTexture then
			MK:SetVertexColor(r, g, b, a);
		else
			MK:SetTexture(r, g, b, a);
		end
		SV.ColorBarMarker = {r,g,b,a};
	end
	self.SetColorBackground = function(self, r,g,b,a)
		BG:SetTexture(r, g, b, a);
		SV.ColorBackground = {r,g,b,a};
	end
	self.SetColorText = function(self, r,g,b,a)
		TX:SetTextColor(r, g, b, a);
		SV.ColorText = {r,g,b,a};
	end
	self.SetBorderVisibility = function(self, val)
		if val then
			Border:Show();
		else
			Border:Hide();
		end
		SV.ShowBorder = val;
	end
	self.SetBorderWidth = function(self, val)
		Border:ClearAllPoints();
		Border:SetPoint("TOPLEFT", Frame, -val, val);
		Border:SetPoint("BOTTOMRIGHT", Frame, val, -val);
	end
	self.OnUpdate = function(self)
		local v = select(15, UnitBuff("player", self.VengeanceName)) or 0;
		-- select(14, UnitBuff("player", "Vengeance")) or 0

		if v == 0 then
			if Frame:IsShown() and not SV.ShowAlways then
				Addon:Hide();
			end
		elseif not Frame:IsShown() then
			Addon:Show();
			self:CalculateVengeanceMax();
		end

		Frame:SetValue(v);
		TX:SetText(v.." - "..ToPercentNumber(v/self.VengeanceMax));
		self.VengeanceValue = v;
		self.VengeanceCurrent = max(self.VengeanceCurrent, v);
		Frame:SetMarkerValue(max(self.VengeancePrev, self.VengeanceCurrent));

		SV.MaxValue = max(v, SV.MaxValue or 0);
	end;
	-- Ya gotta be Insanê to join that bunch *snort*
	Frame:SetScript("OnMouseDown", function(self)
		self:StartMoving();
	end);
	Frame:SetScript("OnMouseUp", function(self)
		self:StopMovingOrSizing();
		SV.FrameRect = {self:GetRect()};
	end);
	Frame.SetMarkerValue = function(self, value)
		local l,h = Frame:GetMinMaxValues();
		MK:ClearAllPoints();
		MK:SetPoint("LEFT", self:GetWidth()*(value/h) - MK:GetWidth()/2, 0);
		MK:SetPoint("TOP");
		MK:SetPoint("BOTTOM");
	end;
	self:RegisterEvent("UNIT_AURA", "OnUpdate");
	do -- initialize settings
		if #SV.FrameRect == 4 then
			local p = SV.FrameRect;
			Frame:SetPoint("BOTTOMLEFT", p[1], p[2])
			Frame:SetWidth(p[3]);
			Frame:SetHeight(p[4]);
		end
		
		if not SV.ShowBorder then
			Border:Hide();
		end
		Frame:EnableMouse(SV.MouseEnabled);

		if SV.ShowAlways then
			self:Show();
		else
			self:Hide();
		end
		
		self:SetTexture(SV.Texture);
		self:SetColorBar(unpack(SV.ColorBar or {1, 1, 0, 1}));
		self:SetColorBarMarker(unpack(SV.ColorBarMarker or {1, 1, 0, 1}));
		self:SetColorText(unpack(SV.ColorText or {.5, .5, .5, 1}));
		self:SetColorBackground(unpack(SV.ColorBackground or {0, 0, 0, 0.5}));
	end
	self:OnMaxHealthUpdate();
end

function Addon:OnMaxHealthUpdate(e)
	self:CalculateVengeanceMax();
	self:OnUpdate();
end

function Addon:CalculateVengeanceMax()
	local hp = UnitHealthMax("player");
	if self.VengeanceMax > 0 then -- exclude short-time bonuses
		self.VengeanceMax = min(self.VengeanceMax, hp);
	else
		self.VengeanceMax = hp;
	end
	self.Frame:SetMinMaxValues(0, self.VengeanceMax);
end

function Addon:AurasChanged(e, ...)
	local s = strjoin(" ", e, tostringall(...));
	DEFAULT_CHAT_FRAME:AddMessage(MAJOR..": "..s);
end

function Addon:Show()
	self.Frame:Show();
	self:RegisterEvent("UNIT_AURA", "OnUpdate");
	self:RegisterEvent("UNIT_MAXHEALTH", "OnMaxHealthUpdate");
	self:RegisterEvent("PLAYER_LEVEL_UP", "OnMaxHealthUpdate");
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED", "OnMaxHealthUpdate");
	self:RegisterEvent("PLAYER_TALENT_UPDATE", "OnMaxHealthUpdate");
	self:RegisterEvent("PLAYER_REGEN_ENABLED");
	self:RegisterEvent("PLAYER_REGEN_DISABLED");
end

function Addon:Hide()
	self:UnregisterEvent("UNIT_MAXHEALTH");
	self:UnregisterEvent("PLAYER_LEVEL_UP");
	self:UnregisterEvent("ACTIVE_TALENT_GROUP_CHANGED");
	self:UnregisterEvent("PLAYER_TALENT_UPDATE");
	self.Frame:Hide();
end

function Addon:PLAYER_REGEN_ENABLED()
    self.VengeancePrev = self.VengeanceCurrent; -- save max vengeance from previous fight
	self.VengeanceCurrent = 0;
end

function Addon:PLAYER_REGEN_DISABLED()
end

--[[ OPTIONS ]]--
local function CreateLocalization()
	local L = LibStub("AceLocale-3.0"):GetLocale(MAJOR, true);
	if L then return L; end
	local locale;
	
	--[[ --------------------- ENGLISH LOCALE STRINGS --------------------- ]]--
	locale = "enUS";
	L = LibStub("AceLocale-3.0"):NewLocale(MAJOR, locale, true);
	L.DESCRIPTION = "Vengeance displays a tank's current level of ... Vengeance! (In a neat bar of course)";
	L.SHOW_ALWAYS = "Always show bar";
	L.SHOW_ALWAYS_DESC = "If checked, the bar will always be visible. Otherwise it will only be visible if you actually have at least one point vengeance.";
	L.BACKGROUND_COLOR = "Background color";
	L.BAR = "Bar";
	L.BAR_COLOR = "Bar color";
	L.BAR_MARKER_COLOR = "Marker color";
	L.BAR_WIDTH = "Width";
	L.BAR_HEIGHT = "Height";
	L.BAR_SHOWBORDER = "Show border";
	L.BORDER_WIDTH = "Border width";
	L.SAMPLE_VALUE = "Sample data";
	L.SAMPLE_VALUE_RESET = "Resets the bar so that sample data is not shown anymore.";
	L.FRAME = "Frame";
	L.FRAME_LOCK = "Lock frame";
	L.FRAME_LOCK_DESC = "Locks the frame (disables mouse interaction)";
	L.TEXTURE = "Texture";
	L.TEXT = "Text";
	L.TEXT_COLOR = "Text color";
	L.TRY = "";
	--[[ --------------------- GERMAN  LOCALE STRINGS --------------------- ]]--
	locale = "deDE";
	L = LibStub("AceLocale-3.0"):NewLocale(MAJOR, locale, true);
	L.DESCRIPTION = "Vengeance zeigt den aktuellen Rachewerte eines Tank-Charakters an.";
	L.SHOW_ALWAYS = "Immer anzeigen";
	L.SHOW_ALWAYS_DESC = "Wenn ausgewählt wird der Balken immer angezeigt, andernfalls wird er nur angezeigt wenn Rache mindestens einen Punkt hat.";
	L.BACKGROUND_COLOR = "Hintergrundfarbe";
	L.BAR = "Balken";
	L.BAR_COLOR = "Vordergrundfarbe";
	L.BAR_MARKER_COLOR = "Markierungsfarbe";
	L.BAR_WIDTH = "Breite";
	L.BAR_HEIGHT = "Höhe";
	L.BAR_SHOWBORDER = "Rahmen anzeigen";
	L.BORDER_WIDTH = "Rahmenbreite";
	L.SAMPLE_VALUE = "BEispielsdaten";
	L.SAMPLE_VALUE_RESET = "Setzt die Anzeige zurück, sodass die Beispieldaten verschwinden.";
	L.FRAME = "Frame";
	L.FRAME_LOCK = "Sperren";
	L.FRAME_LOCK_DESC = "Verhindert Mausinteraktion mit dem Balken";
	L.TEXTURE = "Textur";
	L.TEXT = "Text";
	L.TEXT_COLOR = "Textfarbe";
	L.TRY = "";

	return LibStub("AceLocale-3.0"):GetLocale(MAJOR);
end
	
local function CreateOptionsTable()
	local L = CreateLocalization();
	local SM = LibStub("LibSharedMedia-3.0");
	local SV = VengeanceSavedVariable;
	local Frame = Addon.Frame;
	
	local optionstable = {
		type = "group",
		name = MAJOR,
		childGroups = "tab",
		args = {
			desc = {
				name = L.DESCRIPTION,
				type = "description",
				order = 1,
			},
			visibility = {
				name = L.SHOW_ALWAYS,
				desc = L.SHOW_ALWAYS_DESC,
				order = 2,
				type = "toggle",
				set = function(info, val)
					SV.ShowAlways = val;
					if val then
						Frame:Show();
					else
						Addon:OnUpdate();
					end
				end,
				get = function(info) return SV.ShowAlways; end,
			},
			mouse = {
				name = L.FRAME_LOCK,
				desc = L.FRAME_LOCK_DESC,
				order = 3,
				width = "double",
				type = "toggle",
				set = function(info, val)
					SV.MouseEnabled = not val;
					Frame:EnableMouse(not val);
				end,
				get = function(info) return not SV.MouseEnabled; end,
			},
			try = {
				name = L.TRY,
				order = 4,
				type = "group",
				inline = true,
				args = {
					sample = {
						name = L.SAMPLE_VALUE,
						order = 1,
						type = "execute",
						func = function(info, val)
							Frame:SetMinMaxValues(0,10);
							Frame:SetValue(5);
							Frame:SetMarkerValue(7);
						end,
					},
					samplereset = {
						name = RESET,
						desc = L.SAMPLE_VALUE_RESET,
						order = 2,
						type = "execute",
						width = "half",
						func = function(info, val)
							Addon:OnMaxHealthUpdate();
						end,
					},
				},
			},
			frame = {
				name = L.FRAME,
				order = 7,
				type = "group",
				inline = true,
				args = {
					width = {
						name = L.BAR_WIDTH,
						type = "range",
						order = 31,
						min = 0,
						max = 2000,
						softMax = 500,
						step = 1,
						width = "double",
						set = function(info, val)
							Frame:SetWidth(val);
							SV.FrameRect = {Frame:GetRect()};
						end,
						get = function(info) return SV.FrameRect[3] or 200; end,
					},
					height = {
						name = L.BAR_HEIGHT,
						type = "range",
						order = 32,
						min = 0,
						max = 1000,
						softMax = 200,
						step = 1,
						width = "double",
						set = function(info, val)
							Frame:SetHeight(val);
							SV.FrameRect = {Frame:GetRect()};
						end,
						get = function(info) return SV.FrameRect[4] or 20; end,
					},
					opacity = {
						name = OPACITY,
						type = "range",
						order = 20,
						min = 0,
						max = 1,
						step = .01,
						bigStep = .1,
						set = function(info, val) SV.Opacity = val; Frame:SetAlpha(val); end,
						get = function(info) return SV.Opacity; end,
					},
					borderwidth = {
						name = L.BORDER_WIDTH,
						type = "range",
						order = 21,
						min = 0,
						max = 1000,
						softMax = 10,
						step = .01,
						bigStep = .1,
						set = function(info, val)
							SV.BorderWidth = val;
							Addon:SetBorderWidth(val);
						end,
						get = function(info) return SV.BorderWidth or 3; end,
					},
					border = {
						name = L.BAR_SHOWBORDER,
						type = "toggle",
						order = 22,
						set = function(info, val)
							SV.ShowBorder = val;
							Addon:SetBorderVisibility(val);
						end,
						get = function(info) return SV.ShowBorder; end,
					},
				},
			},
			bar = {
				name = L.BAR,
				order = 8,
				type = "group",
				inline = true,
				args = {
					bartexture = {
						name = L.TEXTURE,
						order = 1,
						type = "select",
						set = function(info, val)
							Addon:SetTexture(SM:List("statusbar")[val]);
						end,
						get = function(info)
							for k,v in pairs(SM:List("statusbar")) do
								if v == SV.Texture then
									return k;
								end
							end
						end,
						values = function(info) return SM:List("statusbar"); end,
					},
					barcolor = {
						name = L.BAR_COLOR,
						order = 3,
						type = "color",
						hasAlpha = true,
						set = function(info, r,g,b,a)
							Addon:SetColorBar(r,g,b,a);
						end,
						get = function(info) return unpack(SV.ColorBar or {1,1,1,1}); end,
					},
					markercolor = {
						name = L.BAR_MARKER_COLOR,
						order = 3,
						type = "color",
						hasAlpha = true,
						set = function(info, r,g,b,a)
							Addon:SetColorBarMarker(r,g,b,a);
						end,
						get = function(info) return unpack(SV.ColorBarMarker or {0,1,1,1}); end,
					},
					backgroundcolor = {
						name = L.BACKGROUND_COLOR,
						order = 4,
						type = "color",
						hasAlpha = true,
						set = function(info, r,g,b,a)
							Addon:SetColorBackground(r,g,b,a);
						end,
						get = function(info) return unpack(SV.ColorBackground or {0,0,0,.5}); end,
					},
				},
			},
			text = {
				name = L.TEXT,
				order = 8,
				type = "group",
				inline = true,
				args = {
					--[[bartexture = {
						name = L.TEXTURE,
						order = 1,
						type = "select",
						set = function(info, val)
							Addon:SetTexture(SM:List("statusbar")[val]);
						end,
						get = function(info)
							for k,v in pairs(SM:List("statusbar")) do
								if v == SV.Texture then
									return k;
								end
							end
						end,
						values = function(info) return SM:List("statusbar"); end,
					},]]
					textcolor = {
						name = L.TEXT_COLOR,
						order = 3,
						type = "color",
						hasAlpha = true,
						set = function(info, r,g,b,a)
							SV.ColorText = {r,g,b,a};
							Addon:SetColorText(r,g,b,a);
						end,
						get = function(info) return unpack(SV.ColorText or {1,1,1,1}); end,
					},
				},
			},
		},
	};
	
	return optionstable;
end

local optionstable = nil;
local function GetOptionsTable()
	if not optionstable then
		optionstable = CreateOptionsTable();
		CreateOptionsTable = nil;
	end
	return optionstable;
end

local AceConfig = LibStub("AceConfig-3.0");
local AceConfigDialog = LibStub("AceConfigDialog-3.0");

AceConfig:RegisterOptionsTable(MAJOR, GetOptionsTable);
optionstablecategory = AceConfigDialog:AddToBlizOptions(MAJOR, nil, nil);
